#include <stdio.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h> 
#include <util/delay.h> 
#include "serialconsole.h"

// Files for input and output through serial port.
static int serial_write(char, FILE *);
static FILE serialPort = FDEV_SETUP_STREAM(serial_write, 0, _FDEV_SETUP_WRITE);

// Calculate the time per bit -9 CPU cycles (which is the length of the for loop in function serial_write)
#define US_PER_BIT 1000000/SOFTSERIAL_BITRATE-9/F_CPU

#if F_CPU/SOFTSERIAL_BITRATE<100
    #warning SOFTSERIAL_BITRATE is to high for this CPU clock frequency
#endif

// Initialize the serial port
void initSerialConsole(void) 
{
    SOFTSERIAL_TXD_INIT;
    SOFTSERIAL_TXD_HIGH;
    // Bind stdout to the serial port
    stdout = &serialPort;
}


// Write a character to the serial port
static int serial_write(char c, FILE *f) 
{
    cli();
    // Send start bit
    SOFTSERIAL_TXD_LOW;
    _delay_us(US_PER_BIT);
    // Send 8 data bits
    for (uint8_t i=0; i<8; i++) 
    {
        if (c & 1)
        {
            SOFTSERIAL_TXD_HIGH;
        }
        else
        {
            SOFTSERIAL_TXD_LOW;
        }
        _delay_us(US_PER_BIT);
        c=c>>1;
    }
    SOFTSERIAL_TXD_HIGH;  
    sei();
    _delay_us(2*US_PER_BIT);
    return 0;
}
